/*
 * 05/23/2003 - 16:36:28
 * 
 * DictionnairePanel.java - Copyright (C) 2003 Dreux Loic dreuxl@free.fr
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.analyse.merise.gui.panel;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.analyse.core.gui.action.BasicAction;
import org.analyse.core.modules.AnalysePanel;
import org.analyse.core.util.GUIUtilities;
import org.analyse.main.Main;
import org.analyse.merise.gui.table.DictionnaireTable;

public class DictionnairePanel extends AnalysePanel
{
    private JPanel toolbar;

    private JComboBox comboBox;

    private JPopupMenu popupTable;

    private BasicAction supprimer;

    private BasicAction monter, descendre;

    private ActionHandler actionHandler;

    private JTable table;

    private DictionnaireTable dictionnaireTable;

    public DictionnairePanel(DictionnaireTable dictionnaireTable)
    {
        super("DICO");

        this.dictionnaireTable = dictionnaireTable;

        this.setBorder(BorderFactory.createEmptyBorder(3, 3, 0, 3));

        comboBox = new JComboBox(dictionnaireTable.getTypes());

        table = new JTable(dictionnaireTable);
        table.getColumn("Type").setCellEditor(new DefaultCellEditor(comboBox));

        table.getColumnModel().getColumn(3).setMaxWidth(100);
        table.getColumnModel().getColumn(4).setMaxWidth(50);
        table.setRowHeight(4, 1);

        actionHandler = new ActionHandler();

        initAction();
        initPopup();
        initToolbar();

        table.addMouseListener(new MouseHandler());

        this.setLayout(new BorderLayout());
        this.add(BorderLayout.CENTER, new JScrollPane(table));
        this.add(BorderLayout.SOUTH, toolbar);
    }

    private void initToolbar()
    {
        toolbar = new JPanel();
        toolbar.setLayout(new FlowLayout(FlowLayout.CENTER));

        toolbar.add(new JButton(monter) {
            {
                setText("");
                addMouseListener(Main.statusbar.getHandler());
            }
        });
        toolbar.add(new JButton(descendre) {
            {
                setText("");
                addMouseListener(Main.statusbar.getHandler());
            }
        });
    }

    private void initAction()
    {
        supprimer = new BasicAction("Supprimer",
                "Supprimer les lignes sélectionnées", "DELETE", GUIUtilities
                        .getImageIcon("Delete.png"), 0, null);
        supprimer.addActionListener(actionHandler);

        monter = new BasicAction(
                "Monter",
                "Inverser le ou les élément(s) sélectionné(s) avec le précédent",
                "UP", GUIUtilities.getImageIcon("Up.png"), 0, null);
        monter.addActionListener(actionHandler);

        descendre = new BasicAction("Descendre",
                "Inverser le ou les élément(s) sélectionné(s) avec le suivant",
                "DOWN", GUIUtilities.getImageIcon("Down.png"), 0, null);
        descendre.addActionListener(actionHandler);
    }

    private void initPopup()
    {
        popupTable = new JPopupMenu();

        popupTable.add(new JMenuItem(monter) {
            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });
        popupTable.add(new JMenuItem(descendre) {
            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });
        popupTable.addSeparator();
        popupTable.add(new JMenuItem(supprimer) {
            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });
    }

    private class MouseHandler extends MouseAdapter
    {
        public void mousePressed(MouseEvent e)
        {
            java.awt.Point point = e.getPoint();

            if (table.getSelectedRowCount() == 0
                    || !table.isRowSelected(table.rowAtPoint(point)))
                table.changeSelection(table.rowAtPoint(point), 0, false, false);
        }

        public void mouseReleased(MouseEvent e)
        {
            //if (e.isPopupTrigger()) Ne marche pas avec le JDK d'IBM
            if (e.getButton() == MouseEvent.BUTTON3)
                popupTable.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    private class ActionHandler implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String action = e.getActionCommand();
            boolean test;

            if (action.equals("DELETE")) {
                String mess;
                if (table.getSelectedRowCount() == 1)
                    mess = "Voulez-vous vraiment supprimer l'élément sélectionné ?";
                else
                    mess = "Voulez-vous vraiment supprimer les "
                            + table.getSelectedRowCount()
                            + " éléments sélectionnés ?";

                test = false;
                for (int i = 0; i < dictionnaireTable.getRowCount(); i++) {
                    if (table.isRowSelected(i) && dictionnaireTable.getUse(i))
                        test = true;

                }
                if (test && table.getSelectedRowCount() == 1)
                    mess += "\nAttention, l'information sélectionnée est présente dans le MCD.";
                else if (test)
                    mess += "\nAttention, certaines informations sélectionnées sont présentes dans le MCD.";
                if (JOptionPane.showConfirmDialog(null, mess, "Analyse",
                        JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
                    return;

                boolean[] row = new boolean[dictionnaireTable.getRowCount() - 1];
                for (int i = 0; i < dictionnaireTable.getRowCount() - 1; i++) {
                    if (table.isRowSelected(i))
                        row[i] = true;
                    else
                        row[i] = false;
                }
                dictionnaireTable.delLines(row);
            } else if (action.equals("UP")) {
                boolean sel = false;
                int start = 0, end = 0;

                for (int i = 0; i < dictionnaireTable.getRowCount(); i++) {
                    if (table.isRowSelected(i) && !sel) {
                        start = i;
                        sel = true;
                    } else if (!table.isRowSelected(i) && sel) {
                        end = i - 1;
                        break;
                    }
                }
                if (dictionnaireTable.swapLines(start, end,
                        DictionnaireTable.UP)) {
                    boolean first = false;
                    for (int i = 0; i < dictionnaireTable.getRowCount(); i++) {
                        if (i >= start - 1 && i <= end - 1) {
                            table.changeSelection(i, 0, false, first);
                            first = true;
                        }
                    }
                }
            } else if (action.equals("DOWN")) {
                boolean sel = false;
                int start = 0, end = 0;

                for (int i = 0; i < dictionnaireTable.getRowCount(); i++) {
                    if (table.isRowSelected(i) && !sel) {
                        start = i;
                        sel = true;
                    } else if (!table.isRowSelected(i) && sel) {
                        end = i - 1;
                        break;
                    }
                }
                if (dictionnaireTable.swapLines(start, end,
                        DictionnaireTable.DOWN)) {
                    boolean first = false;
                    for (int i = 0; i < dictionnaireTable.getRowCount(); i++) {
                        if (i >= start + 1 && i <= end + 1) {
                            table.changeSelection(i, 0, false, first);
                            first = true;
                        }
                    }
                }
            }
        }
    }
}
